@@ -56,7 +56,7 @@ npqreg.call <-
   }
 
 
-npqreg.conbandwidth <-
+npqreg.condbandwidth <-
   function(bws,
            txdat = stop("training data 'txdat' missing"),
            tydat = stop("training data 'tydat' missing"),
@@ -272,7 +272,7 @@ npqreg.default <- function(bws, txdat, tydat, ...){
   ty.str <- ifelse(tydat.named, "ydat = tydat,",
                    ifelse(no.tydat, "", "tydat,"))
   
-  tbw <- eval(parse(text = paste("npcdensbw(",
+  tbw <- eval(parse(text = paste("npcdistbw(",
                       ifelse(bws.named,                             
                              paste(tx.str, ty.str,
                                    "bws = bws, bandwidth.compute = FALSE,"),
