@@ -61,7 +61,7 @@ npindex.formula <-
     tmf <- bws$call[c(1,m)]
     tmf[[1]] <- as.name("model.frame")
     tmf[["formula"]] <- tt
-    umf <- tmf <- eval(tmf)
+    umf <- tmf <- eval(tmf, envir = environment(tt))
 
     tydat <- model.response(tmf)
     txdat <- tmf[, attr(attr(tmf, "terms"),"term.labels"), drop = FALSE]
@@ -69,12 +69,12 @@ npindex.formula <-
     if ((has.eval <- !is.null(newdata))) {
       if (!(has.ey <- succeedWithResponse(tt, newdata)))
         tt <- delete.response(tt)
-      
+
       umf <- emf <- model.frame(tt, data = newdata)
 
       if (has.ey)
         eydat <- model.response(emf)
-      
+
       exdat <- emf[, attr(attr(emf, "terms"),"term.labels"), drop = FALSE]
     }
 
@@ -110,7 +110,7 @@ npindex.default <- function(bws, txdat, tydat, ...){
   no.tydat <- missing(tydat)
 
   ## if bws was passed in explicitly, do not compute bandwidths
-    
+
   if(txdat.named)
     txdat <- toFrame(txdat)
 
@@ -120,9 +120,9 @@ npindex.default <- function(bws, txdat, tydat, ...){
                    ifelse(no.txdat, "", "txdat,"))
   ty.str <- ifelse(tydat.named, "ydat = tydat,",
                    ifelse(no.tydat, "", "tydat,"))
-  
+
   tbw <- eval(parse(text = paste("npindexbw(",
-                      ifelse(bws.named,                             
+                      ifelse(bws.named,
                              paste(tx.str, ty.str,
                                    "bws = bws, bandwidth.compute = FALSE,"),
                              paste(ifelse(no.bws, "", "bws,"), tx.str, ty.str)),
@@ -133,7 +133,7 @@ npindex.default <- function(bws, txdat, tydat, ...){
   ##                            bws = tbw)
 
   repair.args <- c("data", "subset", "na.action")
-  
+
   m.par <- match(repair.args, names(mc), nomatch = 0)
   m.child <- match(repair.args, names(tbw$call), nomatch = 0)
 
@@ -145,7 +145,7 @@ npindex.default <- function(bws, txdat, tydat, ...){
   m.bws.par <- match(c("bws","txdat","tydat"), names(mc), nomatch = 0)
   m.bws.child <- match(c("bws","txdat","tydat"), as.character(tbw$call), nomatch = 0)
   m.bws.union <- (m.bws.par > 0) & (m.bws.child > 0)
-  
+
   tbw$call[m.bws.child[m.bws.union]] <- mc[m.bws.par[m.bws.union]]
 
   environment(tbw$call) <- parent.frame()
@@ -156,13 +156,13 @@ npindex.default <- function(bws, txdat, tydat, ...){
     ty.str <- ",tydat = tydat"
   } else {
     tx.str <- ifelse(txdat.named, ",txdat = txdat","")
-    ty.str <- ifelse(tydat.named, ",tydat = tydat","")    
+    ty.str <- ifelse(tydat.named, ",tydat = tydat","")
     if((!bws.named) && (!txdat.named)){
       ty.str <- ifelse(tydat.named, ",tydat = tydat",
                        ifelse(no.tydat,"",",tydat"))
     }
   }
-  
+
   eval(parse(text=paste("npindex(bws = tbw", tx.str, ty.str, ",...)")))
 }
 
@@ -177,17 +177,15 @@ npindex.sibandwidth <-
            errors = FALSE,
            boot.num = 399, ...) {
 
-  
     no.ex = missing(exdat)
     no.ey = missing(eydat)
-    
 
     ## if no.ex then if !no.ey then ey and tx must match, to get
     ## oos errors alternatively if no.ey you get is errors if
     ## !no.ex then if !no.ey then ey and ex must match, to get
     ## oos errors alternatively if no.ey you get NO errors since we
     ## don't evaluate on the training data
-    
+
     txdat = toFrame(txdat)
 
     if (!(is.vector(tydat) | is.factor(tydat)))
@@ -201,15 +199,15 @@ npindex.sibandwidth <-
 
     if (!no.ex){
       exdat = toFrame(exdat)
-      
+
       if (! txdat %~% exdat )
         stop("'txdat' and 'exdat' are not similar data frames!")
-      
+
       if (!no.ey){
         if (dim(exdat)[1] != length(eydat))
           stop("number of evaluation data 'exdat' and dependent data 'eydat' do not match")
       }
-      
+
     } else if(!no.ey) {
       if (dim(txdat)[1] != length(eydat))
         stop("number of training data 'txdat' and dependent data 'eydat' do not match")
@@ -226,7 +224,6 @@ npindex.sibandwidth <-
     txdat = txdat[goodrows,,drop = FALSE]
     tydat = tydat[goodrows]
 
-
     if (!no.ex){
       goodrows = 1:dim(exdat)[1]
       rows.omit = eval(parse(text=paste('attr(na.omit(data.frame(exdat',
@@ -252,7 +249,6 @@ npindex.sibandwidth <-
     else
       tydat <- as.double(tydat)
 
-
     if (no.ey)
       eydat <- double()
     else {
@@ -266,9 +262,9 @@ npindex.sibandwidth <-
 
     ## re-assign levels in training and evaluation data to ensure correct
     ## conversion to numeric type.
-    
+
     txdat <- adjustLevels(txdat, bws$xdati)
-      
+
     if (!no.ex)
       exdat <- adjustLevels(exdat, bws$xdati)
 
@@ -280,14 +276,13 @@ npindex.sibandwidth <-
 
     ## put the unordered, ordered, and continuous data in their own objects
     ## data that is not a factor is continuous.
-    
+
     txdat = toMatrix(txdat)
 
     if (!no.ex){
       exdat = toMatrix(exdat)
     }
 
-
     ## from this point on txdat and exdat have been recast as matrices
 
     ## TRISTEN - how do we cast txdat here so that matrix multiplication
@@ -298,7 +293,7 @@ npindex.sibandwidth <-
     index <- txdat %*% bws$beta
 
     if(no.ex) {
-      index.eval <- index 
+      index.eval <- index
       exdat <- txdat
       eydat <- tydat
     } else {
@@ -309,23 +304,35 @@ npindex.sibandwidth <-
 
     if(gradients==FALSE) {
 
-      index.mean <- npksum(txdat=index,
-                           tydat=tydat,
-                           exdat=index.eval,
-                           bws=bws$bw)$ksum/
-                             npksum(txdat=index,
-                                    exdat=index.eval,
-                                    bws=bws$bw)$ksum
+      tww <- npksum(txdat=as.matrix(txdat) %*% as.matrix(bws$beta),
+                    tydat=as.matrix(data.frame(tydat,1)),
+                    weights=as.matrix(data.frame(tydat,1)),
+                    exdat=as.matrix(exdat) %*% as.matrix(bws$beta),
+                    bws=bws$bw,
+                    ckertype = bws$ckertype,
+                    ckerorder = bws$ckerorder)$ksum
+
+      denom <- tww[2,2,]
+      denom[which(denom == 0.0)] <- sqrt(.Machine$double.eps)
+      index.mean <- tww[1,2,]/denom
 
       if(!no.ex & (no.ey | residuals)){
-        ## want to evaluate on training data for in sample errors even if evaluation x's
-        ## are different from training but no y's are specified
-        
-        index.tmean <- npksum(txdat=index,
-                              tydat=tydat,
-                              bws=bws$bw)$ksum/
-                                npksum(txdat=index,
-                                       bws=bws$bw)$ksum
+
+        ## want to evaluate on training data for in sample errors even
+        ## if evaluation x's are different from training but no y's
+        ## are specified
+
+        tww <- npksum(txdat=as.matrix(txdat) %*% as.matrix(bws$beta),
+                      tydat=as.matrix(data.frame(tydat,1)),
+                      weights=as.matrix(data.frame(tydat,1)),
+                      bws=bws$bw,
+                      ckertype = bws$ckertype,
+                      ckerorder = bws$ckerorder)$ksum
+
+        denom <- tww[2,2,]
+        denom[which(denom == 0.0)] <- sqrt(.Machine$double.eps)
+        index.tmean <- tww[1,2,]/denom
+
       }
 
     } else if(gradients==TRUE) {
@@ -334,54 +341,81 @@ npindex.sibandwidth <-
                      tydat=tydat,
                      exdat=index.eval,
                      bws=bws$bw,
+                     ckertype = bws$ckertype,
+                     ckerorder = bws$ckerorder,
                      regtype="lc",
                      gradients=TRUE)
 
       index.mean <- model$mean
-      
-      ## index.grad is a matrix, one column for each variable, each equal
-      ## to its coefficient beta_i times the first derivative of the
-      ## local-constant model
-      
-      index.grad <- as.matrix(model$grad)%*%t(as.vector(bws$beta))
+
+      ## index.grad is a 1-column matrix
+
+      index.grad <- model$grad
 
       if(!no.ex & (no.ey | residuals)){
-        ## want to evaluate on training data for in sample errors even if evaluation x's
-        ## are different from training but no y's are specified
 
-        index.tmean <- npreg(txdat=index,
-                             tydat=tydat,
-                             bws=bws$bw,
-                             regtype="lc",
-                             gradients=TRUE)$mean
+        ## Want to evaluate on training data for in sample errors even
+        ## if evaluation x's are different from training but no y's
+        ## are specified. Also, needed for variance-covariance matrix
+        ## (uses on ly the training data)
+
+        model <- npreg(txdat=index,
+                       tydat=tydat,
+                       bws=bws$bw,
+                       ckertype = bws$ckertype,
+                       ckerorder = bws$ckerorder,
+                       regtype="lc",
+                       gradients=TRUE)
+
+        index.tmean <- model$mean
+
+        index.tgrad <- model$grad
+
       }
 
-
     }
 
-    if (no.ex) index.tmean <- index.mean
+    if (no.ex) {
+      index.tmean <- index.mean
+    }
 
-    ## June 3 2010, jracine, added vcov methods... thanks to Juan
-    ## Carlos Escanciano <jescanci@indiana.edu> for pushing me on this
-    ## for the Klein and Spady estimator... use index.tmean (training
-    ## X) and index (tx) - need gradients == TRUE in order for this to
-    ## work
+    if (no.ex & gradients) {
+      index.tgrad <- index.grad
+    }
+
+    ## 5/3/2010, jracine, added vcov methods... thanks to Juan Carlos
+    ## Escanciano <jescanci@indiana.edu> for pushing me on this for
+    ## the Klein and Spady estimator... use index.tmean, index.tgrad
+    ## (training X) - need gradients == TRUE in order for this to
+    ## work.
 
     if(bws$method == "ichimura" & gradients == TRUE) {
 
-      ## First row & column of covariance matrix are zero due to
-      ## identification condition that beta_1=0. Note the n^{-1} in
-      ## the E and the \sqrt{n} in the normalization of \hat\beta will
-      ## cancel.
+      ## First row & column of covariance matrix `Bvcov' are zero due
+      ## to identification condition that beta_1=0. Note the n n^{-1}
+      ## n in V^{-1}\Sigma V^{-1} and the \sqrt{n} in the
+      ## normalization of \hat\beta will cancel.
 
       q <- ncol(txdat)
       Bvcov <- matrix(0,q,q)
       dimnames(Bvcov) <- list(bws$xnames,bws$xnames)
 
+      ## Use the weight matrix so we can compute all expectations with
+      ## only one call to npksum (the kernel arguments x\beta do not
+      ## change, only the j for X_{ij} in E(X_{ij}|X_i'\beta)
+
       W <- txdat[,-1,drop=FALSE]
 
-      tyindex <- npksum(txdat = index, tydat = rep(1,length(tydat)), weights = W, bws = bws$bw)$ksum
-      tindex <- npksum(txdat = index, bws = bws$bw)$ksum
+      tyindex <- npksum(txdat = index,
+                        tydat = rep(1,length(tydat)),
+                        weights = W,
+                        bws = bws$bw,
+                        ckertype = bws$ckertype,
+                        ckerorder = bws$ckerorder)$ksum
+
+      tindex <- npksum(txdat = index, bws = bws$bw,
+                       ckertype = bws$ckertype,
+                       ckerorder = bws$ckerorder)$ksum
 
       ## Need to trap case where k-1=1... ksum will return a 1 D
       ## array, need a 1 x n matrix
@@ -395,39 +429,44 @@ npindex.sibandwidth <-
       ## Need to trap case where k-1=1..., sapply will return a
       ## vector, need a 1 x n matrix
 
-      if(is.vector(xmex)) xmex <- matrix(xmex,nrow=1,ncol=length(xmex))
-
-      dg.db.sq <- (W*index.grad[,1])^2
-
-      dg.db.sq.xmex <- sapply(1:length(tydat),function(i){dg.db.sq[i,]*xmex[,i]})      
+      if(is.vector(xmex)) {
+        dg.db.xmex <- matrix(index.tgrad[,1]*xmex,nrow=1,ncol=length(xmex))
+      } else {
+        dg.db.xmex <- index.tgrad[,1]*xmex
+      }
 
       uhat <- tydat - index.tmean ## Training y and training mean
 
-      Vinv <- solve(dg.db.sq.xmex%*%t(xmex))
+      Vinv <- solve(dg.db.xmex%*%t(dg.db.xmex))
 
-      Sigma <- ((uhat^2)*dg.db.sq.xmex)%*%t(xmex)
+      Sigma <- (uhat*dg.db.xmex)%*%t(uhat*dg.db.xmex)
 
       Bvcov[-1,-1] <- Vinv %*% Sigma %*% Vinv
-    
-      dimnames(Bvcov) <- list(bws$xnames,bws$xnames)      
+
+      dimnames(Bvcov) <- list(bws$xnames,bws$xnames)
+
+      ## Now export this in an S3 method...
 
     } else if(bws$method == "kleinspady" & gradients == TRUE) {
 
       ## We divide by P(1-P) so test for P=0 or 1...
+
       keep <- which(index.tmean < 1 & index.tmean > 0)
-      dg.db <- txdat[,-1,drop=FALSE]*index.grad[,1]
+      dg.db <- txdat[,-1,drop=FALSE]*index.tgrad[,1]
+
       ## First row & column of covariance matrix are zero due to
       ## identification condition that beta_1=0. Note the n^{-1} in
       ## the E and the \sqrt{n} in the normalization of \hat\beta will
       ## cancel.
+
       q <- ncol(txdat)
       Bvcov <- matrix(0,q,q)
       Bvcov[-1,-1] <- solve(t(dg.db[keep,])%*%(dg.db[keep,]/(index.tmean[keep]*
         (1-index.tmean[keep]))))
 
-      dimnames(Bvcov) <- list(bws$xnames,bws$xnames)      
+      dimnames(Bvcov) <- list(bws$xnames,bws$xnames)
 
-      ## Now export these in an S3 method...
+      ## Now export this in an S3 method...
 
     }
 
@@ -446,7 +485,9 @@ npindex.sibandwidth <-
           txdat = rindex,
           tydat = tydat[indices],
           exdat = index.eval,
-          bws = bws$bw)[c('mean','grad')]
+          bws = bws$bw,
+          ckertype = bws$ckertype,
+          ckerorder = bws$ckerorder)[c('mean','grad')]
 
         c(model$mean, model$grad, mean(model$grad))
       }
@@ -455,8 +496,12 @@ npindex.sibandwidth <-
       boofun = function(data, indices){
         rindex = txdat[indices,] %*% bws$beta
         npksum(txdat = rindex, tydat = tydat[indices], exdat = index.eval,
-               bws = bws$bw)$ksum/
-                 npksum(txdat = rindex, exdat = index.eval, bws=bws$bw)$ksum
+               bws = bws$bw,
+               ckertype = bws$ckertype,
+               ckerorder = bws$ckerorder)$ksum/
+                 npksum(txdat = rindex, exdat = index.eval, bws=bws$bw,
+                        ckertype = bws$ckertype,
+                        ckerorder = bws$ckerorder)$ksum
       }
     }
 
@@ -471,10 +516,9 @@ npindex.sibandwidth <-
         index.gerr = matrix(data = 0, ncol = ncol(txdat), nrow = length(index.eval))
         index.gerr[,] = sqrt(diag(cov(boot.out$t[,(length(index.eval)+1):(2*length(index.eval))])))
 
-        for (i in ncol(txdat))
+        for (i in 1:ncol(txdat))
           index.gerr[,i] = abs(bws$beta[i])*index.gerr[,i]
 
-
         index.mgerr = sd(boot.out$t[,2*length(index.eval)+1])
         index.mgerr = abs(bws$beta)*index.mgerr
       }
@@ -503,34 +547,31 @@ npindex.sibandwidth <-
       index.pred =
         if (!no.ey) round(index.mean)
         else round(index.tmean)
-      
+
       confusion.matrix =
         table(if (!no.ey) eydat else tydat,
               index.pred, dnn=c("Actual", "Predicted"))
-      
+
       CCR.overall <- sum(diag(confusion.matrix))/sum(confusion.matrix)
       CCR.byoutcome <- diag(confusion.matrix)/rowSums(confusion.matrix)
 
-
       fit.mcfadden <- confusion.matrix/sum(confusion.matrix)
-      
+
       fit.mcfadden <- sum(diag(fit.mcfadden)) -
         (sum(fit.mcfadden^2)-sum(diag(fit.mcfadden)^2))
 
-
       strgof = "confusion.matrix = confusion.matrix, CCR.overall = CCR.overall,
            CCR.byoutcome =  CCR.byoutcome, fit.mcfadden = fit.mcfadden,"
       strres = ""
     }
 
-
     eval(parse(text=paste(
                  "singleindex(bws = bws, index = index.eval, mean = index.mean,",
                  ifelse(errors,"merr = index.merr,",""),
-                 ifelse(gradients,"grad = index.grad, mean.grad = colMeans(index.grad), betavcov = Bvcov,",""),
+                 ifelse(gradients,"grad = index.grad, mean.grad = mean(index.grad), betavcov = Bvcov,",""),
                  ifelse(errors & gradients,"gerr = index.gerr, mean.gerr = index.mgerr,",""),
                  strres,
                  "ntrain = nrow(txdat),", strgof,
                  "trainiseval = no.ex, residuals = residuals, gradients = gradients)")))
-  
+
   }
