@@ -1,5 +1,5 @@
 .onAttach <- function (lib, pkg) {
-	packageStartupMessage("Nonparametric Kernel Methods for Mixed Datatypes (version 0.40-13)", domain = NULL,  appendLF = TRUE)
+	packageStartupMessage("Nonparametric Kernel Methods for Mixed Datatypes (version 0.50-1)\n[vignette(\"np_faq\",package=\"np\") provides answers to frequently asked questions]", domain = NULL,  appendLF = TRUE)
 
   if(is.null(options('np.messages')$np.messages))
     options(np.messages = TRUE)
@@ -8,6 +8,6 @@
     options(np.tree = FALSE)
 
 }
-.Last.lib <- function (lpath){
+.onUnload <- function (lpath){
   library.dynam.unload("np", libpath=lpath) 
 }
